/*   https://forum.caliban.be/viewtopic.php?t=154
MPU 650 SDA 21 - SCL 22 - VCC - 19
Neopixel 8X  23
BP 26-GND*/

#include <WiFi.h>
#include <ESPmDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include "driver/rtc_io.h"
#include <Adafruit_MPU6050.h>
#include <Adafruit_Sensor.h>
#include <Wire.h>
#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
#include <avr/power.h>
#endif

#define BUTTON_PIN_BITMASK 0x200000000  // 2^33 in hex
#define LED_PIN 23
#define LED_COUNT 8
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

int MPU_EN = 19;
Adafruit_MPU6050 mpu;

const char* ssid = "XXX";
const char* password = "XXX";

float acc = 0;
float gyro = 0;
int seuil_acc = 100;
int count = 3;
int DelayAutoOff = 1800000;  //3600000
unsigned long previousTimeRainbow = 0;
unsigned long currentTime = 0;
unsigned long previousTime = 0;
int firstPixelHue = 0;
const int buttonPin = 26;
int buttonState = 1;


void setup(void) {
  Serial.begin(115200);
  delay(100);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.waitForConnectResult() != WL_CONNECTED) {
    Serial.println("Connection Failed! Rebooting...");
    delay(5000);
    ESP.restart();
  }
  ArduinoOTA.setHostname("MOON");
  ArduinoOTA.setPassword("MOON");
  ArduinoOTA
    .onStart([]() {
      String type;
      if (ArduinoOTA.getCommand() == U_FLASH)
        type = "sketch";
      else
        type = "filesystem";
      Serial.println("Start updating " + type);
    })
    .onEnd([]() {
      Serial.println("\nEnd");
    })
    .onProgress([](unsigned int progress, unsigned int total) {
      Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
    })
    .onError([](ota_error_t error) {
      Serial.printf("Error[%u]: ", error);
      if (error == OTA_AUTH_ERROR) Serial.println("Auth Failed");
      else if (error == OTA_BEGIN_ERROR) Serial.println("Begin Failed");
      else if (error == OTA_CONNECT_ERROR) Serial.println("Connect Failed");
      else if (error == OTA_RECEIVE_ERROR) Serial.println("Receive Failed");
      else if (error == OTA_END_ERROR) Serial.println("End Failed");
    });

  ArduinoOTA.begin();
  Serial.println("Ready");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
  pinMode(buttonPin, INPUT_PULLUP);
  rtc_gpio_pullup_en(GPIO_NUM_26);
  esp_sleep_enable_ext0_wakeup(GPIO_NUM_26, 0);
  #if defined(__AVR_ATtiny85__) && (F_CPU == 16000000)
    clock_prescale_set(clock_div_1);
  #endif
  strip.begin();             // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();              // Turn OFF all pixels ASAP
  strip.setBrightness(255);  // Set BRIGHTNESS to about 1/5 (max = 255)
  pinMode(MPU_EN, OUTPUT);
  digitalWrite(MPU_EN, HIGH);
  Serial.println("Adafruit MPU6050 test!");
  if (!mpu.begin()) {
    Serial.println("Failed to find MPU6050 chip");
    while (1) {
      delay(10);
    }
  }
  Serial.println("MPU6050 Found!");
  mpu.setAccelerometerRange(MPU6050_RANGE_16_G);
  mpu.setGyroRange(MPU6050_RANGE_2000_DEG);
  mpu.setFilterBandwidth(MPU6050_BAND_260_HZ);
  delay(100);
}

void loop() {

  ArduinoOTA.handle();
  buttonState = digitalRead(buttonPin);
  if (buttonState == LOW) {
    Serial.println("Go to Sleep");
    Blink();
    Sleep();
  }
  sensors_event_t a, g, temp;
  mpu.getEvent(&a, &g, &temp);
  acc = abs(a.acceleration.x + a.acceleration.y + a.acceleration.z);
  gyro = abs(g.gyro.x + g.gyro.y + g.gyro.z); 
  if (acc > seuil_acc) {
    count = count + 1;
    count %= 4;  // si count > 4 count = 0
    Serial.println(count);
    Blink();
    previousTime = currentTime;
  }
  currentTime = millis();
  switch (count) {
    case 0:
      { //Changement de couleur via gyroscope
        if (gyro > 1) {
          previousTime = currentTime;
          Rainbow();
        }
        break;
      }
    case 1:
      { //Changement de couleur progressive
        if ((currentTime - previousTimeRainbow) > 100) {
          previousTimeRainbow = currentTime;
          Rainbow();
        }
        break;
      }
    case 2:
      { //Blanc Froid
        colorWipe(strip.Color(255, 255, 255), 0); 
        break;
      }
    case 3:
      { //Blanc Chaud
        colorWipe(strip.Color(245, 127, 23), 0);
        break;
      }
      
  }
  if ((currentTime - previousTime) > DelayAutoOff) {
    Sleep();
  }
}
void colorWipe(uint32_t color, int wait) {
  for (int i = 0; i < strip.numPixels(); i++) {
    strip.setPixelColor(i, color);
    strip.show();
    delay(wait);
  }
}
void Rainbow() {
  for (int a = 0; a < 8; a++) {
    strip.setPixelColor(a, strip.gamma32(strip.ColorHSV(firstPixelHue)));  // Set pixel 'c' to value 'color'
  }
  firstPixelHue += 65536 / 360;
  firstPixelHue %= 65536;
  strip.show();
}

void Sleep() {
  colorWipe(strip.Color(0, 0, 0), 0);
  Serial.println("Going to sleep now");
  esp_deep_sleep_start();
}

void Blink() {
  colorWipe(strip.Color(255, 0, 0), 0);
  strip.show();
  delay(300);
  colorWipe(strip.Color(0, 255, 0), 0);
  strip.show();
  delay(300);
  colorWipe(strip.Color(0, 0, 2550), 0);
  strip.show();
  delay(300);
}